@@ -4,38 +4,58 @@ package provide wheredoesthisgo 0.1
 # a place to temporarily store things until they find a home or go away
 
 proc open_file {filename} {
+    if {$filename == ""} { return }
     set directory [file normalize [file dirname $filename]]
     set basename [file tail $filename]
     if { ! [file exists $filename]} {
-        ::pdwindow::post [format [_ "Ignoring '%s': doesn't exist"] $filename]
+        ::pdwindow::post [format [_ "ignoring '%s': doesn't exist"] $filename]
+        ::pdwindow::post "\n"
         # remove from recent files
         ::pd_guiprefs::update_recentfiles $filename true
         return
     }
+    set ::fileopendir $directory
     if {[regexp -nocase -- "\.(pd|pat|mxt)$" $filename]} {
         ::pdtk_canvas::started_loading_file [format "%s/%s" $basename $filename]
         pdsend "pd open [enquote_path $basename] [enquote_path $directory]"
         # now this is done in pd_guiprefs
         ::pd_guiprefs::update_recentfiles $filename
     } else {
-        ::pdwindow::post [format [_ "Ignoring '%s': doesn't look like a Pd-file"] $filename]
+        ::pdwindow::post [format [_ "ignoring '%s': doesn't look like a Pd file"] $filename]
+        ::pdwindow::post "\n"
     }
 }
 
 # ------------------------------------------------------------------------------
 # procs for panels (openpanel, savepanel)
 
-proc pdtk_openpanel {target localdir} {
+proc pdtk_openpanel {target localdir {mode 0}} {
     if {! [file isdirectory $localdir]} {
         if { ! [file isdirectory $::fileopendir]} {
             set ::fileopendir $::env(HOME)
         }
         set localdir $::fileopendir
     }
-    set filename [tk_getOpenFile -initialdir $localdir]
-    if {$filename ne ""} {
-        set ::fileopendir [file dirname $filename]
-        pdsend "$target callback [enquote_path $filename]"
+    # 0: file, 1: directory, 2: multiple files
+    switch $mode {
+        0 { set result [tk_getOpenFile -initialdir $localdir] }
+        1 { set result [tk_chooseDirectory -initialdir $localdir] }
+        2 { set result [tk_getOpenFile -multiple 1 -initialdir $localdir] }
+        default { ::pdwindow::error "bad value for 'mode' argument" }
+    }
+    if {$result ne ""} {
+        if { $mode == 2 } {
+            # 'result' is a list
+            set ::fileopendir [file dirname [lindex $result 0]]
+            set args {}
+            foreach path $result {
+                lappend args [enquote_path $path]
+            }
+            pdsend "$target callback [join $args]"
+        } else {
+            set ::fileopendir [expr {$mode == 0 ? [file dirname $result] : $result}]
+            pdsend "$target callback [enquote_path $result]"
+        }
     }
 }
 
@@ -88,6 +108,21 @@ proc tkcanvas_name {mytoplevel} {
     return "$mytoplevel.c"
 }
 
+# ------------------------------------------------------------------------------
+# window helpers
+
+# position one window over another
+proc position_over_window {child parent} {
+    if {![winfo exists $parent]} {return}
+    # use internal tk::PlaceWindow http://wiki.tcl.tk/534
+    # so fallback if not available
+    if {[catch {tk::PlaceWindow $child widget $parent}]} {
+        set x [expr [winfo x $parent] + ([winfo width $parent] / 2) - ([winfo reqwidth $child] / 2)]
+        set y [expr [winfo y $parent] + ([winfo height $parent] / 2) - ([winfo reqheight $child] / 2)]
+        wm geometry $child "+${x}+${y}"
+    }
+}
+
 # ------------------------------------------------------------------------------
 # quoting functions
 
@@ -116,7 +151,14 @@ proc enquote_path {message} {
 #enquote a string to send it to Pd.  Blow off semi and comma; alias spaces
 #we also blow off "{", "}", "\" because they'll just cause bad trouble later.
 proc unspace_text {x} {
-    set y [string map {" " "_" ";" "" "," "" "{" "" "}" "" "\\" ""} $x]
+    set y [string map {" " {\ } ";" "" "," "" "{" "" "}" "" "\\" ""} $x]
+    if {$y eq ""} {set y "empty"}
+    concat $y
+}
+
+#dequote a string received from Pd.
+proc respace_text {x} {
+    set y [string map {{\ } " "} $x]
     if {$y eq ""} {set y "empty"}
     concat $y
 }
